(window.webpackJsonp=window.webpackJsonp||[]).push([[9],{qnT9:function(t,n,s){"use strict";s.r(n),s.d(n,"SettingsPageModule",(function(){return r}));var e=s("ofXK"),c=s("fXoL"),i=s("WZmd");let b=(()=>{class t{constructor(t){this.settings=t,this.backendStatus="offline"}ngOnInit(){this.backendUrlSubscription=this.settings.backend.subscribe({next:t=>this.backendUrl=t}),this.backendStatusSubscription=this.settings.isConnected.subscribe({next:t=>this.backendStatus=t?"online":"offline"})}ngOnDestroy(){this.backendUrlSubscription.unsubscribe(),this.backendStatusSubscription.unsubscribe()}}return t.\u0275fac=function(n){return new(n||t)(c.Jb(i.a))},t.\u0275cmp=c.Db({type:t,selectors:[["app-settings-page"]],decls:8,vars:2,template:function(t,n){1&t&&(c.Pb(0,"h3"),c.zc(1,"Backend URL"),c.Ob(),c.Pb(2,"p"),c.zc(3),c.Ob(),c.Pb(4,"h3"),c.zc(5,"Backend Status"),c.Ob(),c.Pb(6,"p"),c.zc(7),c.Ob()),2&t&&(c.yb(3),c.Ac(n.backendUrl),c.yb(4),c.Ac(n.backendStatus))},styles:[""]}),t})();var a=s("tyNb");let r=(()=>{class t{}return t.\u0275mod=c.Hb({type:t}),t.\u0275inj=c.Gb({factory:function(n){return new(n||t)},imports:[[e.c,a.f.forChild([{path:"",component:b,pathMatch:"full"}])]]}),t})()}}]);