(window.webpackJsonp=window.webpackJsonp||[]).push([[8],{ry1Y:function(e,t,c){"use strict";c.r(t),c.d(t,"HomePageModule",(function(){return i}));var n=c("ofXK"),r=c("fXoL"),s=c("ws30");let a=(()=>{class e{constructor(e){this.exerciseService=e,this.allTracks=[]}ngOnInit(){this.exerciseService.getTrackIds().then(e=>this.allTracks=e)}}return e.\u0275fac=function(t){return new(t||e)(r.Jb(s.a))},e.\u0275cmp=r.Db({type:e,selectors:[["app-home-page"]],decls:3,vars:3,template:function(e,t){1&e&&(r.Pb(0,"pre"),r.zc(1),r.Zb(2,"json"),r.Ob()),2&e&&(r.yb(1),r.Bc("  ",r.ac(2,1,t.allTracks),"\n"))},pipes:[n.i],styles:[""]}),e})();var o=c("tyNb");let i=(()=>{class e{}return e.\u0275mod=r.Hb({type:e}),e.\u0275inj=r.Gb({factory:function(t){return new(t||e)},imports:[[n.c,o.f.forChild([{path:"",component:a,pathMatch:"full"}])]]}),e})()}}]);