(window.webpackJsonp=window.webpackJsonp||[]).push([[0],{Yv6K:function(c,e,r){"use strict";r.d(e,"a",(function(){return b}));var s=r("ofXK"),t=r("8MG2"),n=r("3Pt+"),i=r("tyNb"),o=r("fXoL");let b=(()=>{class c{}return c.\u0275mod=o.Hb({type:c}),c.\u0275inj=o.Gb({factory:function(e){return new(e||c)},imports:[[s.c,n.g,i.f,t.a]]}),c})()},pgGW:function(c,e,r){"use strict";r.d(e,"a",(function(){return d}));var s=r("fXoL"),t=r("aLtf"),n=r("8MG2"),i=r("ofXK"),o=r("tyNb");const b=function(c){return["/","e",c]};function l(c,e){if(1&c&&(s.Pb(0,"clr-dg-row"),s.Pb(1,"clr-dg-cell"),s.zc(2),s.Ob(),s.Pb(3,"clr-dg-cell"),s.zc(4),s.Ob(),s.Pb(5,"clr-dg-cell"),s.Kb(6,"clr-icon"),s.Ob(),s.Pb(7,"clr-dg-cell"),s.Kb(8,"clr-icon"),s.Ob(),s.Pb(9,"clr-dg-cell"),s.Pb(10,"button",1),s.zc(11),s.Ob(),s.Ob(),s.Ob()),2&c){const c=e.$implicit,r=s.Yb();s.yb(2),s.Ac(c.order),s.yb(2),s.Ac(c.id),s.yb(2),s.zb("shape",c.mandatory?"check":"times"),s.yb(2),s.zb("shape",r.exerciseProgress[c.id]?"check":"times"),s.yb(2),s.dc("routerLink",s.hc(6,b,c.id)),s.yb(1),s.Bc(" ",r.exerciseProgress[c.id]?"Restart":"Start"," ")}}let d=(()=>{class c{constructor(c){this.progress=c,this.exercises=[],this.exerciseProgress={}}ngOnInit(){this.exercises=this.exercises.sort((c,e)=>c.order-e.order),this.exercises.forEach(c=>{const e=this.progress.solveStatus(c.id);e&&(this.exerciseProgress[c.id]=e)})}}return c.\u0275fac=function(e){return new(e||c)(s.Jb(t.a))},c.\u0275cmp=s.Db({type:c,selectors:[["app-exercise-list"]],inputs:{exercises:"exercises"},decls:11,vars:1,consts:[[4,"ngFor","ngForOf"],[1,"btn","btn-link",3,"routerLink"]],template:function(c,e){1&c&&(s.Pb(0,"clr-datagrid"),s.Pb(1,"clr-dg-column"),s.zc(2,"#"),s.Ob(),s.Pb(3,"clr-dg-column"),s.zc(4,"ID"),s.Ob(),s.Pb(5,"clr-dg-column"),s.zc(6,"Mandatory"),s.Ob(),s.Pb(7,"clr-dg-column"),s.zc(8,"Done"),s.Ob(),s.Kb(9,"clr-dg-column"),s.xc(10,l,12,8,"clr-dg-row",0),s.Ob()),2&c&&(s.yb(10),s.dc("ngForOf",e.exercises))},directives:[n.h,n.X,n.Z,n.T,n.ab,n.j,n.W,i.n,n.q,n.Y,n.i,n.V,n.w,o.c],styles:[""]}),c})()}}]);